/* Interface include. */
#include "component_loader.h"

/* ISR DB. */
#include "isr_db.h"

/* Initialize data section. */
static void initialize_data_section( uint32_t data_rom_start,
                                     uint32_t data_ram_start,
                                     uint32_t data_ram_end );

/* Initialize BSS section. */
static void initialize_bss_section( uint32_t bss_ram_start,
                                    uint32_t bss_ram_end );

/* Load all the functions exported by the component. */
static void load_functions( uint32_t functions_table_start,
                            component_t * p_component );

/* Install all the ISRs required by the component. */
static void install_isrs( uint32_t isr_table_start );
/*-----------------------------------------------------------*/

static void initialize_data_section( uint32_t data_rom_start,
                                     uint32_t data_ram_start,
                                     uint32_t data_ram_end )
{
    uint32_t * p_src = ( uint32_t * ) data_rom_start;
    uint32_t * p_dst = ( uint32_t * ) data_ram_start;
    uint32_t * p_dst_end = ( uint32_t * ) data_ram_end;

    while( p_dst != p_dst_end )
    {
        *p_dst++ = *p_src++;
    }
}
/*-----------------------------------------------------------*/

static void initialize_bss_section( uint32_t bss_ram_start,
                                    uint32_t bss_ram_end )
{
    uint32_t * p_dst = ( uint32_t * ) bss_ram_start;
    uint32_t * p_dst_end = ( uint32_t * ) bss_ram_end;

    while( p_dst != p_dst_end )
    {
        *p_dst++ = 0;
    }
}
/*-----------------------------------------------------------*/

static void install_isrs( uint32_t isr_table_start )
{
    uint32_t isr_id, isr_address;
    uint32_t * p_isr_table = ( uint32_t * )isr_table_start;

    for( ; ; )
    {
        isr_id = *p_isr_table++;
        isr_address = *p_isr_table++;

        if( ( isr_id == 0 ) && ( isr_address == 0 ) )
        {
            /* This is the last entry. */
            break;
        }
        else
        {
            install_isr( isr_id, isr_address );
        }
    }
}
/*-----------------------------------------------------------*/

static void load_functions( uint32_t functions_table_start,
                            component_t * p_component )
{
    uint32_t i, function_id, function_address;
    uint32_t * p_functions_table = ( uint32_t * )functions_table_start;

    for( i = 0; i < p_component->functions_table_size; i++ )
    {
        function_id = *p_functions_table++;
        function_address = *p_functions_table++;

        if( ( function_id == 0 ) && ( function_address == 0 ) )
        {
            /* This is the last entry. */
            break;
        }
        else
        {
            if( function_id < p_component->functions_table_size )
            { 
                p_component->p_functions_table[ function_id ] = function_address;
            }
            else
            {
                /* Handle error. */
            }
        }
    }

    /* Update the actual number of functions loaded. */
    p_component->functions_table_size= i;
}
/*-----------------------------------------------------------*/

void load_component( uint32_t component_id,
                     uint32_t start_address,
                     component_t * p_component )
{
    uint32_t component_id_at_header;
    uint32_t data_rom_start, data_ram_start, data_ram_end;
    uint32_t bss_ram_start, bss_ram_end;
    uint32_t entry_point;
    uint32_t functions_table_start, isr_table_start;
    uint32_t * p_component_header_addr = ( uint32_t * ) start_address;

    component_id_at_header = *p_component_header_addr++;

    /* Component ID expected by the loader must match to what is declared by the
     * component. */
    if( component_id_at_header == component_id )
    {
        data_rom_start = *p_component_header_addr++;
        data_ram_start = *p_component_header_addr++;
        data_ram_end = *p_component_header_addr++;
        bss_ram_start = *p_component_header_addr++;
        bss_ram_end = *p_component_header_addr++;
        entry_point = *p_component_header_addr++;
        functions_table_start = *p_component_header_addr++;
        isr_table_start = *p_component_header_addr++;

        initialize_data_section( data_rom_start, data_ram_start, data_ram_end );
        initialize_bss_section( bss_ram_start, bss_ram_end );

        load_functions( functions_table_start, p_component );
        install_isrs( isr_table_start );

        p_component->entry_point = ( entry_point_t )entry_point;
    }
    else
    {
        /* Handle error. */
    }
}
/*-----------------------------------------------------------*/
