/* Standard includes. */
#include <stdlib.h>

/* ISR DB. */
#include "isr_db.h"

/* Component loader. */
#include "component_loader.h"

/* Interface include. */
#include "components_db.h"

#define DEFINE_COMPONENT( start_address, function_table, function_table_size ) \
    { ( uint32_t ) start_address, { NULL, function_table, function_table_size } }

/* Function table sizes for various components. */
#define BSP_MAX_EXPORTED_FUNCTIONS           5
#define FREERTOS_MAX_EXPORTED_FUNCTIONS         5
#define APPLICATION_MAX_EXPORTED_FUNCTIONS      5

/* Function tables for various components. */
static uint32_t bsp_function_table[ BSP_MAX_EXPORTED_FUNCTIONS ];
static uint32_t freeRTOS_function_table[ FREERTOS_MAX_EXPORTED_FUNCTIONS ];
static uint32_t application_function_table[ APPLICATION_MAX_EXPORTED_FUNCTIONS ];
/*-----------------------------------------------------------*/

typedef struct component_db_entry
{
    uint32_t start_address;
    component_t component;
} component_db_entry_t;
/*-----------------------------------------------------------*/

static component_db_entry_t component_db[] =
{
    DEFINE_COMPONENT( 0x08010000, bsp_function_table, BSP_MAX_EXPORTED_FUNCTIONS ),
    DEFINE_COMPONENT( 0x08020000, freeRTOS_function_table, FREERTOS_MAX_EXPORTED_FUNCTIONS ),
    DEFINE_COMPONENT( 0x08030000, application_function_table, APPLICATION_MAX_EXPORTED_FUNCTIONS ),
};

/* Total number of components. */
static uint32_t num_components = 0;

/* Entry point which MUST be defined by only one component. */
static entry_point_t entry_point = NULL;
/*-----------------------------------------------------------*/

/* Return address of a function. */
uint32_t get_proc_address( uint32_t component_id,
                           uint32_t function_id ) __attribute__ ((section(".get_proc_address")));
/*-----------------------------------------------------------*/

void load_all_components( void )
{
    uint32_t component_id;

    num_components = sizeof( component_db ) / sizeof( component_db[ 0 ] );

    /* Initialize the ISR DB. */
    initialize_isr_db();

    /* Load all the components one by one. */
    for( component_id = 0; component_id < num_components; component_id++ )
    {
        load_component( component_id,
                        component_db[ component_id ].start_address,
                        &( component_db[ component_id ].component ) );

        if( component_db[ component_id ].component.entry_point != NULL )
        {
            if( entry_point == NULL )
            {
                entry_point = component_db[ component_id ].component.entry_point;
            }
            else
            {
                /* Handle error - more than one components have defined entry
                 * point. */
            }
        }
    }

    /* Start using ISR DB. */
    apply_isr_db();
}
/*-----------------------------------------------------------*/

void invoke_entry_point( void )
{
    if( entry_point != NULL )
    {
        entry_point();
    }
}
/*-----------------------------------------------------------*/

uint32_t get_proc_address( uint32_t component_id, uint32_t function_id )
{
    uint32_t addr = 0;

    if( ( component_id < num_components ) &&
        ( function_id < component_db[ component_id ].component.functions_table_size ) )
    {
        addr = component_db[ component_id ].component.p_functions_table[ function_id ];
    }

    return addr;
}
/*-----------------------------------------------------------*/
